@@ -84,48 +84,26 @@ OPTION_MAXIMISE_STR
 }
 
 static void
-do_file (const char *filename, double *reference, size_t reference_size,
-         int *nobj_p, const signed char * minmax, bool maximise_all_flag)
+do_file(const char * filename, double * restrict reference, size_t reference_size,
+        int * restrict nobj_p, const int * restrict minmax, bool maximise_all_flag)
 {
-    double *data = NULL;
-    int *cumsizes = NULL;
+    double * data = NULL;
+    int * cumsizes = NULL;
     int nruns = 0;
-    int nobj = *nobj_p;
-
-    handle_read_data_error(
-        read_double_data (filename, &data, &nobj, &cumsizes, &nruns), filename);
-    if (!filename)
+    robust_read_double_data(filename, &data, nobj_p, &cumsizes, &nruns, /*union_flag=*/false);
+    if (filename == NULL)
         filename = stdin_name;
+    dimension_t nobj = (dimension_t) *nobj_p;
 
-    if (!additive_flag && !all_positive(data, cumsizes[nruns - 1], (dimension_t) nobj)) {
+    if (!additive_flag && !all_positive(data, cumsizes[nruns - 1], nobj)) {
         errprintf("cannot calculate multiplicative epsilon indicator with non-positive values when reading '%s'.", filename);
         exit(EXIT_FAILURE);
     }
 
-    char *outfilename = NULL;
-    FILE *outfile = stdout;
-    if (filename != stdin_name && suffix) {
-        outfilename = m_strcat(filename, suffix);
-        outfile = fopen (outfilename, "w");
-        if (outfile == NULL) {
-            errprintf ("%s: %s\n", outfilename, strerror(errno));
-            exit (EXIT_FAILURE);
-        }
-    }
-#if 0
-    if (union_flag) {
-        cumsizes[0] = cumsizes[nruns - 1];
-        nruns = 1;
-    }
-#endif
-    ASSUME(nobj > 1 && nobj < 256);
-    dimension_t dim = (dimension_t) nobj;
-    /* Default minmax if not set yet.  */
-    bool free_minmax = false;
-    if (minmax == NULL) {
-        minmax = maximise_all_flag ? minmax_maximise(dim) : minmax_minimise(dim);
-        free_minmax = true;
-    }
+    const char * outfilename = NULL;
+    FILE * outfile = fopen_outfile(&outfilename, filename, suffix);
+    // Default minmax if not set yet.
+    bool free_minmax = minmax_alloc(&minmax, maximise_all_flag, nobj);
 
     if (verbose_flag)
         printf("# file: %s\n", filename);
@@ -134,10 +112,10 @@ do_file (const char *filename, double *reference, size_t reference_size,
         // double time_elapsed = 0;
         //Timer_start ();
         double epsilon = (additive_flag)
-            ? epsilon_additive_minmax(minmax, dim,
+            ? epsilon_additive_minmax(minmax, nobj,
                                       &data[nobj * cumsize], cumsizes[n] - cumsize,
                                       reference, reference_size)
-            : epsilon_mult_minmax(minmax, dim,
+            : epsilon_mult_minmax(minmax, nobj,
                                   &data[nobj * cumsize], cumsizes[n] - cumsize,
                                   reference, reference_size);
         //        time_elapsed = Timer_elapsed_virtual ();
@@ -152,28 +130,15 @@ do_file (const char *filename, double *reference, size_t reference_size,
          */
     }
 
-    if (outfilename) {
-        if (verbose_flag)
-            fprintf (stderr, "# %s -> %s\n", filename, outfilename);
-        fclose (outfile);
-        free (outfilename);
-    }
-    free (data);
-    free (cumsizes);
-    if (free_minmax) free( (void *) minmax);
-    *nobj_p = nobj;
+    fclose_outfile(outfile, filename, outfilename, verbose_flag);
+    free(data);
+    free(cumsizes);
+    if (free_minmax) free((void *) minmax);
 }
 
 int main(int argc, char *argv[])
 {
-    bool check_flag = true;
-    double *reference = NULL;
-    size_t reference_size = 0;
-    const signed char *minmax = NULL;
-    bool maximise_all_flag = false;
-    int nobj = 0, tmp_nobj = 0;
-
-    /* see the man page for getopt_long for an explanation of these fields */
+    // See the man page for getopt_long for an explanation of these fields.
     static const char short_options[] = "hVvqamMr:s:o:";
     static const struct option long_options[] = {
         {"help",       no_argument,       NULL, 'h'},
@@ -193,6 +158,13 @@ int main(int argc, char *argv[])
 
     set_program_invocation_short_name(argv[0]);
 
+    bool check_flag = true;
+    double * reference = NULL;
+    size_t reference_size = 0;
+    const int * minmax = NULL;
+    bool maximise_all_flag = false;
+    int nobj = 0;
+
     int opt; /* it's actually going to hold a char */
     int longopt_index;
     while (0 < (opt = getopt_long(argc, argv, short_options,
@@ -219,17 +191,7 @@ int main(int argc, char *argv[])
             break;
 
         case 'r': // --reference
-            reference_size = read_reference_set(&reference, optarg, &tmp_nobj);
-            if (reference == NULL || reference_size == 0) {
-                errprintf ("invalid reference set '%s", optarg);
-                exit(EXIT_FAILURE);
-            }
-            if (nobj == 0) {
-                nobj = tmp_nobj;
-            } else if (tmp_nobj != nobj) {
-                errprintf ("number of objectives in --obj (%d) and reference set (%d) do not match", nobj, tmp_nobj);
-                exit(EXIT_FAILURE);
-            }
+            reference_size = read_reference_set(optarg, &reference, &nobj);
             break;
 
         case 's': // --suffix
@@ -262,9 +224,9 @@ int main(int argc, char *argv[])
         minmax = maximise_all_flag ? minmax_maximise((dimension_t) nobj) : minmax_minimise((dimension_t) nobj);
     }
     if (check_flag) {
-        /* Ensure the reference set is nondominated.  */
+        // Ensure the reference set is nondominated.
         size_t prev_reference_size = reference_size;
-        reference_size = filter_dominated_set(reference, nobj, reference_size, minmax);
+        reference_size = filter_dominated_set(reference, reference_size, (dimension_t) nobj, minmax);
         if (prev_reference_size > reference_size)
             warnprintf("removed %zd dominated points from the reference set",
                        prev_reference_size - reference_size);
@@ -274,10 +236,10 @@ int main(int argc, char *argv[])
         exit(EXIT_FAILURE);
     }
     int numfiles = argc - optind;
-    if (numfiles < 1) { /* Read stdin.  */
-        do_file (NULL, reference, reference_size, &nobj, minmax, maximise_all_flag);
+    if (numfiles < 1) { // Read stdin.
+        do_file(NULL, reference, reference_size, &nobj, minmax, maximise_all_flag);
     } else if (numfiles == 1) {
-        do_file (argv[optind], reference, reference_size, &nobj, minmax, maximise_all_flag);
+        do_file(argv[optind], reference, reference_size, &nobj, minmax, maximise_all_flag);
     } else {
         int k;
         /* FIXME: Calculate the nondominated front among all input
@@ -285,7 +247,7 @@ int main(int argc, char *argv[])
 #if 0
         if (reference == NULL) {
             reference_size =
-                calculate_nondominated (&reference, data, nobj, cumsizes[nruns-1],
+                calculate_nondominated(&reference, data, nobj, cumsizes[nruns-1],
                                         minmax);
             write_sets (stderr, reference, nobj, &reference_size, 1);
         }
@@ -306,6 +268,6 @@ int main(int argc, char *argv[])
     }
 
     free(reference);
-    free((void*)minmax);
+    free((void *)minmax);
     return EXIT_SUCCESS;
 }
