@@ -43,9 +43,6 @@ static inline uint32_t mt19937_next(mt19937_state *state) {
   return y;
 }
 
-extern void mt19937_init_by_array(mt19937_state *state, uint32_t *init_key,
-                                  int key_length);
-
 static inline uint64_t mt19937_next64(mt19937_state *state) {
   return (uint64_t)mt19937_next(state) << 32 | mt19937_next(state);
 }
