@@ -22,32 +22,40 @@ typedef struct avl_node_t {
 enum objs_agree_t { AGREE_MINIMISE = -1, AGREE_NONE = 0, AGREE_MAXIMISE = 1 };
 
 static inline enum objs_agree_t
-check_all_minimize_maximize(const signed char * restrict minmax, dimension_t dim)
+check_all_minimize_maximize(const int * restrict minmax, dimension_t dim)
 {
-    bool all_minimize = true, all_maximize = true;
-    for (dimension_t d = 0; d < dim; d++) {
-        if (minmax[d] < 0) {
-            all_maximize = false;
-        } else if (minmax[d] > 0) {
-            all_minimize = false;
-        } else {
-            all_minimize = false;
-            all_maximize = false;
-            break;
-        }
-    }
+    bool all_minimize = (minmax[0] < 0);
+    bool all_maximize = (minmax[0] > 0);
     assert(!all_maximize || !all_minimize);
-    if (all_minimize) return AGREE_MINIMISE;
-    if (all_maximize) return AGREE_MAXIMISE;
+    if (all_minimize) {
+        for (dimension_t d = 1; d < dim; d++) {
+            if (minmax[d] >= 0)
+                return AGREE_NONE;
+        }
+        return AGREE_MINIMISE;
+    }
+    if (all_maximize) {
+        for (dimension_t d = 1; d < dim; d++) {
+            if (minmax[d] <= 0)
+                return AGREE_NONE;
+        }
+        return AGREE_MAXIMISE;
+    }
     return AGREE_NONE;
 }
 
-/* Convert from bool vector to minmax vector.  */
-static inline signed char *
+/** Convert from bool vector to minmax vector.
+
+    minmax needs to be an int (int64 in 64 bits) so that GCC can vectorize
+    loops that are conditional on minmax[d].  Using 'signed char' does not work
+    in some targets, e.g, -march=tigerlake.
+
+ */
+static inline int *
 minmax_from_bool(const bool * restrict maximise, dimension_t nobj)
 {
     ASSUME(nobj > 0 && nobj < 128);
-    signed char * minmax = malloc(nobj * sizeof(*minmax));
+    int * minmax = malloc(nobj * sizeof(*minmax));
     for (dimension_t k = 0; k < nobj; k++) {
         minmax[k] = maximise[k] ? AGREE_MAXIMISE : AGREE_MINIMISE;
     }
@@ -64,29 +72,41 @@ new_bool_maximise(dimension_t nobj, bool maximise_all)
     return maximise;
 }
 
-static inline const signed char *
-default_minmax(dimension_t nobj, signed char default_value)
+static inline const int *
+default_minmax(dimension_t nobj, int default_value)
 {
     ASSUME(nobj > 0 && nobj < 128);
     ASSUME(default_value == AGREE_MINIMISE || default_value == AGREE_MAXIMISE);
-    signed char * minmax = malloc(nobj * sizeof(*minmax));
+    int * minmax = malloc(nobj * sizeof(*minmax));
     for (dimension_t i = 0; i < nobj; i++)
         minmax[i] = default_value;
     return minmax;
 }
 
-static inline const signed char *
+static inline const int *
 minmax_minimise(dimension_t nobj)
 {
     return default_minmax(nobj, AGREE_MINIMISE);
 }
 
-static inline const signed char *
+static inline const int *
 minmax_maximise(dimension_t nobj)
 {
     return default_minmax(nobj, AGREE_MAXIMISE);
 }
 
+static inline bool
+minmax_alloc(const int * restrict * minmax, bool maximise_all_flag,
+             dimension_t d)
+{
+    if (*minmax == NULL) {
+        *minmax = maximise_all_flag ? minmax_maximise(d) : minmax_minimise(d);
+        return true;
+    }
+    return false;
+}
+
+
 // FIXME: It would be more efficient to default to false (using calloc) and
 // change to true.
 static inline bool *
@@ -99,8 +119,8 @@ nondom_init (size_t size)
 }
 
 static inline const double *
-force_agree_minimize(const double * restrict points, dimension_t dim, size_t size,
-                     const signed char * restrict minmax,
+force_agree_minimize(const double * restrict points, size_t size, dimension_t dim,
+                     const int * restrict minmax,
                      _attr_maybe_unused const enum objs_agree_t agree)
 {
     assert(agree != AGREE_MINIMISE);
@@ -134,7 +154,7 @@ generate_sorted_pp_2d(const double *points, size_t size)
         p[k] = points + 2 * k;
 
     // Sort in ascending lexicographic order from the last dimension.
-    qsort(p, size, sizeof(*p), &cmp_double_asc_rev_2d);
+    qsort(p, size, sizeof(*p), cmp_ppdouble_asc_rev_2d);
     return p;
 }
 
@@ -356,9 +376,9 @@ find_nondominated_set_3d_(const double * restrict points, size_t size, bool * re
 }
 
 static inline size_t
-find_nondominated_set_agree_(const double * restrict points, dimension_t dim, size_t size,
+find_nondominated_set_agree_(const double * restrict points, size_t size, dimension_t dim,
                              bool * restrict nondom, const bool keep_weakly,
-                             const enum objs_agree_t agree, const signed char * restrict minmax)
+                             const enum objs_agree_t agree, const int * restrict minmax)
 {
     ASSUME(dim > 3);
     ASSUME(agree == AGREE_MINIMISE || agree == AGREE_MAXIMISE || agree == AGREE_NONE);
@@ -410,36 +430,34 @@ find_nondominated_set_agree_(const double * restrict points, dimension_t dim, si
 
             assert(dom_k ^ dom_j); // At least one but not both can be removed.
 
-            if (unlikely(nondom == NULL)) {
-                return dom_k ? k : j;
-            }
-            if (dom_j) {
-                assert(nondom[j]);
-                nondom[j] = false;
-                new_size--;
+            size_t last_dom_pos = dom_j ? j : k;
+            if (unlikely(nondom == NULL))
+                return last_dom_pos;
+
+            new_size--; // Something dominated.
+            assert(nondom[last_dom_pos]);
+            nondom[last_dom_pos] = false;
+            if (dom_j)
                 break;
-            }
-            nondom[k] = false;
-            new_size--;
         }
     }
     return new_size;
 }
 
 static inline size_t
-find_dominated_point_agree_(const double * restrict points, dimension_t dim, size_t size,
+find_dominated_point_agree_(const double * restrict points, size_t size, dimension_t dim,
                             const bool keep_weakly, const enum objs_agree_t agree,
-                            const signed char * restrict minmax)
+                            const int * restrict minmax)
 {
-    return find_nondominated_set_agree_(points, dim, size, /* nondom=*/NULL,
+    return find_nondominated_set_agree_(points, size, dim, /* nondom=*/NULL,
                                         keep_weakly, agree, minmax);
 }
 
 /* Stop as soon as one dominated point is found and return its position.
 **/
 static inline size_t
-find_dominated_point_(const double * restrict points, dimension_t dim, size_t size,
-                      const signed char * restrict minmax, enum objs_agree_t agree,
+find_dominated_point_(const double * restrict points, size_t size, dimension_t dim,
+                      const int * restrict minmax, enum objs_agree_t agree,
                       const bool keep_weakly)
 {
     ASSUME(minmax != NULL);
@@ -448,7 +466,7 @@ find_dominated_point_(const double * restrict points, dimension_t dim, size_t si
 
     ASSUME(dim >= 2);
     if (dim <= 3) {
-        const double *pp = force_agree_minimize(points, dim, size, minmax, agree);
+        const double *pp = force_agree_minimize(points, size, dim, minmax, agree);
         size_t res;
         if (dim == 2) {
             res = find_dominated_2d_(pp, size, keep_weakly);
@@ -456,7 +474,7 @@ find_dominated_point_(const double * restrict points, dimension_t dim, size_t si
             res = find_dominated_3d_(pp, size, keep_weakly);
         }
         if (pp != points)
-            free((void*)pp);
+            free((void *) pp);
         return res;
     }
 
@@ -468,14 +486,14 @@ find_dominated_point_(const double * restrict points, dimension_t dim, size_t si
     switch (agree) {
       case AGREE_NONE:
           return find_dominated_point_agree_(
-              points, dim, size, keep_weakly, AGREE_NONE, minmax);
+              points, size, dim, keep_weakly, AGREE_NONE, minmax);
       case AGREE_MINIMISE:
           return find_dominated_point_agree_(
-              points, dim, size, keep_weakly, AGREE_MINIMISE, /*minmax=*/NULL);
+              points, size, dim, keep_weakly, AGREE_MINIMISE, /*minmax=*/NULL);
       case AGREE_MAXIMISE:
           return find_dominated_point_agree_(
-              points, dim, size, keep_weakly, AGREE_MAXIMISE, /*minmax=*/NULL);
-      default:
+              points, size, dim, keep_weakly, AGREE_MAXIMISE, /*minmax=*/NULL);
+      default: // LCOV_EXCL_LINE # nocov
           unreachable();
     }
 }
@@ -484,8 +502,8 @@ find_dominated_point_(const double * restrict points, dimension_t dim, size_t si
    nondominated points.
 **/
 static inline size_t
-find_nondominated_set_(const double * restrict points, dimension_t dim, size_t size,
-                       const signed char * restrict minmax, enum objs_agree_t agree,
+find_nondominated_set_(const double * restrict points, size_t size, dimension_t dim,
+                       const int * restrict minmax, enum objs_agree_t agree,
                        bool * nondom, const bool keep_weakly)
 {
     if (size < 2)
@@ -496,7 +514,7 @@ find_nondominated_set_(const double * restrict points, dimension_t dim, size_t s
     ASSUME(nondom != NULL);
 
     if (dim <= 3) {
-        const double *pp = force_agree_minimize (points, dim, size, minmax, agree);
+        const double *pp = force_agree_minimize (points, size, dim, minmax, agree);
         size_t res;
         if (dim == 2) {
             res = find_nondominated_set_2d_(pp, size, nondom, keep_weakly);
@@ -504,7 +522,7 @@ find_nondominated_set_(const double * restrict points, dimension_t dim, size_t s
             res = find_nondominated_set_3d_(pp, size, nondom, keep_weakly);
         }
         if (pp != points)
-            free((void*)pp);
+            free((void *) pp);
         return res;
     }
 
@@ -519,68 +537,68 @@ find_nondominated_set_(const double * restrict points, dimension_t dim, size_t s
     switch (agree) {
       case AGREE_NONE:
           return find_nondominated_set_agree_(
-              points, dim, size, nondom, keep_weakly, AGREE_NONE, minmax);
+              points, size, dim, nondom, keep_weakly, AGREE_NONE, minmax);
       case AGREE_MINIMISE:
           return find_nondominated_set_agree_(
-              points, dim, size, nondom, keep_weakly, AGREE_MINIMISE, /*minmax=*/NULL);
+              points, size, dim, nondom, keep_weakly, AGREE_MINIMISE, /*minmax=*/NULL);
       case AGREE_MAXIMISE:
           return find_nondominated_set_agree_(
-              points, dim, size, nondom, keep_weakly, AGREE_MAXIMISE, /*minmax=*/NULL);
-      default:
+              points, size, dim, nondom, keep_weakly, AGREE_MAXIMISE, /*minmax=*/NULL);
+      default: // LCOV_EXCL_LINE # nocov
           unreachable();
     }
 }
 
 static inline size_t
-find_dominated_point_agree(const double * restrict points, int dim, size_t size,
-                           const signed char * restrict minmax,
+find_dominated_point_agree(const double * restrict points, size_t size, dimension_t dim,
+                           const int * restrict minmax,
                            enum objs_agree_t agree)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-    return find_dominated_point_(points, (dimension_t) dim, size, minmax,
-                                 agree, /* keep_weakly = */false);
+    ASSUME(dim >= 2);
+    return find_dominated_point_(points, size, dim, minmax, agree,
+                                 /* keep_weakly = */false);
 }
 
 static inline size_t
-find_dominated_point(const double * restrict points, int dim, size_t size,
-                     const signed char * restrict minmax)
+find_dominated_point(const double * restrict points, size_t size, dimension_t dim,
+                     const int * restrict minmax)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-    return find_dominated_point_(points, (dimension_t) dim, size, minmax,
+    ASSUME(dim >= 2);
+    return find_dominated_point_(points, size, dim, minmax,
                                  AGREE_NONE, /* keep_weakly = */false);
 }
 
 static inline size_t
-find_weakly_dominated_point(const double * restrict points, int dim, size_t size,
+find_weakly_dominated_point(const double * restrict points, size_t size, dimension_t dim,
                             const bool * restrict maximise)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-    const signed char * minmax = minmax_from_bool(maximise, (dimension_t) dim);
-    size_t pos = find_dominated_point_(points, (dimension_t) dim, size, minmax,
+    ASSUME(dim >= 2);
+    const int * minmax = minmax_from_bool(maximise, dim);
+    size_t pos = find_dominated_point_(points, size, dim, minmax,
                                        AGREE_NONE, /* keep_weakly = */false);
     free((void *)minmax);
     return pos;
 }
 
 static inline size_t
-find_nondominated_set_agree(const double * restrict points, int dim, size_t size,
-                            const signed char * restrict minmax, const signed char agree,
+find_nondominated_set_agree(const double * restrict points, size_t size, dimension_t dim,
+                            const int * restrict minmax, const int agree,
                             bool * restrict nondom)
 {
-    ASSUME(dim >= 2 && dim <= 32);
+    ASSUME(dim >= 2);
     ASSUME(agree == AGREE_MINIMISE || agree == AGREE_MAXIMISE || agree == AGREE_NONE);
-    return find_nondominated_set_(points, (dimension_t) dim, size, minmax,
+    return find_nondominated_set_(points, size, dim, minmax,
                                   (enum objs_agree_t) agree, nondom,
                                   /* keep_weakly = */false);
 }
 
 static inline size_t
-find_nondominated_set(const double * restrict points, dimension_t dim, size_t size,
-                      const signed char * restrict minmax, bool * restrict nondom)
+find_nondominated_set(const double * restrict points, size_t size, dimension_t dim,
+                      const int * restrict minmax, bool * restrict nondom)
 {
-    ASSUME(dim >= 2 && dim <= 32);
+    ASSUME(dim >= 2);
     size_t new_size = find_nondominated_set_(
-        points, dim, size, minmax, AGREE_NONE, nondom,
+        points, size, dim, minmax, AGREE_NONE, nondom,
         /* keep_weakly = */false);
 
     if (new_size > size || new_size == 0 || new_size == SIZE_MAX) { /* This can't happen.  */
@@ -593,23 +611,24 @@ find_nondominated_set(const double * restrict points, dimension_t dim, size_t si
 }
 
 static inline size_t
-find_weak_nondominated_set(const double * restrict points, dimension_t dim, size_t size,
-                           const signed char * restrict minmax,
+find_weak_nondominated_set(const double * restrict points, size_t size, dimension_t dim,
+                           const int * restrict minmax,
                            bool * restrict nondom)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-    return find_nondominated_set_(points, dim, size, minmax, AGREE_NONE, nondom,
+    ASSUME(dim >= 2);
+    return find_nondominated_set_(points, size, dim, minmax, AGREE_NONE, nondom,
                                   /* keep_weakly = */true);
 }
 
 static inline size_t
-find_weak_nondominated_set_minimise(const double * restrict points, dimension_t dim,
-                                    size_t size, bool * restrict nondom)
+find_weak_nondominated_set_minimise(const double * restrict points,
+                                    size_t size, dimension_t dim,
+                                    bool * restrict nondom)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-    const signed char * minmax = minmax_minimise(dim);
+    ASSUME(dim >= 2);
+    const int * minmax = minmax_minimise(dim);
     ASSUME(minmax != NULL);
-    size_t new_size = find_weak_nondominated_set(points, dim, size, minmax,
+    size_t new_size = find_weak_nondominated_set(points, size, dim, minmax,
                                                  nondom);
     free((void *)minmax);
     return new_size;
@@ -617,14 +636,13 @@ find_weak_nondominated_set_minimise(const double * restrict points, dimension_t
 
 
 static inline size_t
-get_nondominated_set (double **pareto_set_p,
-                      const double *points, int dim, size_t size,
-                      const signed char *minmax)
+get_nondominated_set(double **pareto_set_p,
+                     const double * restrict points, size_t size, dimension_t dim,
+                     const int * restrict minmax)
 {
-    ASSUME(dim >= 2 && dim <= 32);
-
+    ASSUME(dim >= 2);
     bool * nondom = nondom_init(size);
-    size_t new_size = find_nondominated_set(points, (dimension_t) dim, size, minmax, nondom);
+    size_t new_size = find_nondominated_set(points, size, dim, minmax, nondom);
     double * pareto_set = malloc(new_size * dim * sizeof(*pareto_set));
 
     if (new_size < size) {
@@ -645,20 +663,20 @@ get_nondominated_set (double **pareto_set_p,
 }
 
 static inline size_t
-filter_dominated_set(double * restrict points, int dim, size_t size,
-                     const signed char * restrict minmax)
+filter_dominated_set(double * restrict points, size_t size, dimension_t dim,
+                     const int * restrict minmax)
 {
-    ASSUME(dim >= 2 && dim <= 32);
+    ASSUME(dim >= 2);
     ASSUME(size > 0);
 
     bool * nondom = nondom_init(size);
-    size_t new_size = find_nondominated_set(points, (dimension_t) dim, size, minmax, nondom);
+    size_t new_size = find_nondominated_set(points, size, dim, minmax, nondom);
 
     if (new_size < size) {
         size_t k = 0;
         while (nondom[k]) k++; // Find first dominated.
         size_t n = k;
-        while (k < new_size) {
+        while (k < new_size) { // Otherwise, all the nondominated are at the top.
             do {
                 n++;
             } while (!nondom[n]); // Find next nondominated.
@@ -671,67 +689,66 @@ filter_dominated_set(double * restrict points, int dim, size_t size,
 }
 
 static inline bool *
-is_nondominated_minmax(const double * restrict data, dimension_t nobj, size_t npoint,
-                       const signed char * restrict minmax, bool keep_weakly)
+is_nondominated_minmax(const double * restrict data, size_t npoint, dimension_t nobj,
+                       const int * restrict minmax, bool keep_weakly)
 {
     bool * nondom = nondom_init(npoint);
-    find_nondominated_set_(data, nobj, npoint, minmax, AGREE_NONE, nondom, keep_weakly);
+    find_nondominated_set_(data, npoint, nobj, minmax, AGREE_NONE, nondom, keep_weakly);
     return nondom;
 }
 
 static inline bool *
-is_nondominated_minimise(const double * data, dimension_t nobj, size_t npoint,
+is_nondominated_minimise(const double * restrict data, size_t npoint, dimension_t nobj,
                          bool keep_weakly)
 {
-    const signed char * minmax = minmax_minimise(nobj);
-    bool * nondom = is_nondominated_minmax(data, nobj, npoint, minmax,
+    const int * minmax = minmax_minimise(nobj);
+    bool * nondom = is_nondominated_minmax(data, npoint, nobj, minmax,
                                            keep_weakly);
     free((void *) minmax);
     return nondom;
 }
 
 _attr_maybe_unused static bool *
-is_nondominated(const double * restrict data, int nobj, size_t npoint,
+is_nondominated(const double * restrict data, size_t npoint, dimension_t nobj,
                 const bool * restrict maximise, bool keep_weakly)
 {
-    ASSUME(nobj >= 2 && nobj <= 32);
-    const signed char * minmax = minmax_from_bool(maximise, (dimension_t) nobj);
-    bool * nondom = is_nondominated_minmax(data, (dimension_t) nobj, npoint,
-                                           minmax, keep_weakly);
+    ASSUME(nobj >= 2);
+    const int * minmax = minmax_from_bool(maximise, nobj);
+    bool * nondom = is_nondominated_minmax(data, npoint, nobj, minmax,
+                                           keep_weakly);
     free((void *)minmax);
     return nondom;
 }
 
 static inline void
-agree_objectives (double * restrict points, int dim, int size,
-                  const signed char * restrict minmax, const enum objs_agree_t agree)
+agree_objectives (double * restrict points, size_t size, dimension_t dim,
+                  const int * restrict minmax, const enum objs_agree_t agree)
 {
-    for (int d = 0; d < dim; d++)
+    for (dimension_t d = 0; d < dim; d++)
         if ((agree > 0 && minmax[d] < 0)
             || (agree < 0 && minmax[d] > 0))
-            for (int k = 0; k < size; k++)
+            for (size_t k = 0; k < size; k++)
                 points[k * dim + d] = -(points[k * dim + d]);
 }
 
 
 static inline void
-normalise(double * restrict points, int dim, int size,
-          const signed char * restrict minmax, signed char agree,
+normalise(double * restrict points, size_t size, dimension_t dim,
+          const int * restrict minmax, int agree,
           const double lower_range, const double upper_range,
           const double * restrict lbound, const double * restrict ubound)
 {
     const double range = upper_range - lower_range;
     double * diff = malloc(dim * sizeof(*diff));
-    int k, d;
-    for (d = 0; d < dim; d++) {
+    for (dimension_t d = 0; d < dim; d++) {
         diff[d] = ubound[d] - lbound[d];
         if (diff[d] == 0.0) // FIXME: Should we use approximate equality?
             diff[d] = 1; // FIXME: Do we need to handle agree?
     }
 
-    for (k = 0; k < size; k++) {
+    for (size_t k = 0; k < size; k++) {
         double * p = points + k * dim;
-        for (d = 0; d < dim; d++)
+        for (dimension_t d = 0; d < dim; d++)
             if ((agree > 0 && minmax[d] < 0) || (agree < 0 && minmax[d] > 0))
                 p[d] = lower_range + range * (ubound[d] + p[d]) / diff[d];
             else
@@ -742,20 +759,20 @@ normalise(double * restrict points, int dim, int size,
 }
 
 _attr_maybe_unused static void
-agree_normalise (double * restrict data, int nobj, int npoint,
+agree_normalise (double * restrict data, size_t npoint, dimension_t nobj,
                  const bool * restrict maximise,
                  const double lower_range, const double upper_range,
                  const double * restrict lbound, const double * restrict ubound)
 {
-    const signed char * minmax = minmax_from_bool(maximise, (dimension_t)nobj);
+    const int * minmax = minmax_from_bool(maximise, nobj);
     // We have to make the objectives agree before normalisation.
     // FIXME: Do normalisation and agree in one step.
-    agree_objectives(data, nobj, npoint, minmax, AGREE_MINIMISE);
-    normalise(data, nobj, npoint, minmax, AGREE_MINIMISE,
+    agree_objectives(data, npoint, nobj, minmax, AGREE_MINIMISE);
+    normalise(data, npoint, nobj, minmax, AGREE_MINIMISE,
               lower_range, upper_range, lbound, ubound);
     free ((void *)minmax);
 }
 
-int * pareto_rank(const double * restrict points, size_t size, int dim) _attr_malloc;
+int * pareto_rank(const double * restrict points, size_t size, dimension_t dim) _attr_malloc;
 
 #endif /* NONDOMINATED_H */
